@@ -2,18 +2,19 @@ import json
 import math
 import bitmex
 import time
-import datetime
 import uuid
 import pickle
+import sys
 
 from bravado.exception import HTTPUnauthorized, HTTPBadRequest, HTTPNotFound
 
 
 class BitmexClient:
-    def __init__(self, api_key=None, api_secret=None, test=True):
+    def __init__(self, api_key=None, api_secret=None, testnet=True):
         self.client = bitmex.bitmex(
-            api_key=api_key, api_secret=api_secret, test=test)
+            api_key=api_key, api_secret=api_secret, test=testnet)
         self._last_currentprice = {}
+        self.testnet = testnet
 
     def __call__(self):
         return self.client
@@ -55,9 +56,9 @@ class BitmexClient:
     def get_current_price(self, symbol):
         try:
             price = self.client.Trade.Trade_get(symbol=symbol,
-                                            count=2,
-                                            reverse=True,
-                                            ).result()[0][0]['price']
+                                                count=2,
+                                                reverse=True,
+                                                ).result()[0][0]['price']
             self._last_currentprice[symbol] = price
             return price
         except (IndexError, HTTPBadRequest) as e:
@@ -102,10 +103,11 @@ class BitmexClient:
 
 
 class BitmexOrder:
-    def __init__(self, client, settings_path, magic='algo-trader'):
+    def __init__(self, client, settings, settings_path, magic='algo-trader'):
         self.client = client
-        self.magic = magic
+        self.settings = settings
         self._settings_path = settings_path
+        self.magic = magic
 
         self.props = {}
 
@@ -135,58 +137,76 @@ class BitmexOrder:
             cp = self.client.get_current_price(symbol)
             if self.WaitStop:
                 time.sleep(0.5)
-                if self.props[symbol].qty > 0 and cp >= self.props[symbol].entry and self.client.open_contracts(symbol)!= 0:
+                if self.props[symbol].qty > 0 and cp >= self.props[symbol].entry and self.client.open_contracts(symbol) != 0:
                     self.props[symbol].wait_stop = False
                     self.props[symbol].open = True
                     print("Long Position has been opened, amending Stoploss to",
-                        self.props[symbol].SL, flush=True)
+                          self.props[symbol].SL, flush=True)
                     self.stoploss_order()
                 elif self.props[symbol].qty < 0 and cp <= self.props[symbol].entry and self.client.open_contracts(symbol) != 0:
                     self.props[symbol].wait_stop = False
                     self.props[symbol].open = True
                     print("Short Position has been opened, amending Stoploss to",
-                        self.props[symbol].SL, flush=True)
+                          self.props[symbol].SL, flush=True)
                     self.stoploss_order()
-            elif self.Open:
+            elif self.props[symbol].open:
                 if self.props[symbol].qty > 0 and cp <= self.props[symbol].SL and self.client.open_contracts(symbol) == 0:
                     self.props[symbol].open = False
                     self.props[symbol].qty = 0
                     print("Long Position closed. {} Contracts from {} to {}. PnL in Points: {}".format(
-                        self.props[symbol].qty, self.props[symbol].entry, self.props[symbol].SL, self.bitmex_decimals(cp-self.props[symbol].entry)), flush=True)
+                        self.props[symbol].qty, self.props[symbol].entry, self.props[symbol].SL, self.price_decimals(symbol, cp-self.props[symbol].entry)), flush=True)
                 elif self.props[symbol].qty < 0 and cp >= self.props[symbol].SL and self.client.open_contracts(symbol) == 0:
                     self.props[symbol].open = False
                     self.props[symbol].qty = 0
                     print("Short Position closed. {} Contracts from {} to {}. PnL in Points: {}".format(
-                        -self.props[symbol].qty, self.props[symbol].entry, self.props[symbol].SL, self.bitmex_decimals(cp-self.props[symbol].entry)), flush=True)
+                        -self.props[symbol].qty, self.props[symbol].entry, self.props[symbol].SL, self.price_decimals(symbol, cp-self.props[symbol].entry)), flush=True)
                 else:
                     self.props[symbol].open = False
 
     def generate_id(self, symbol, _type='stop'):
         return "{}_{}_{}_{}".format(self.magic, symbol, str(uuid.uuid4().fields[-1])[:5], _type)
 
-    def bitmex_decimals(self, price):
-        # for entry price calculation [in XBTUSD]
-        return round(price * 2) / 2
+    def price_decimals(self, symbol, price):
+        if symbol == 'XBUTSD':
+            return round(price * 2) / 2
+        elif symbol == 'ETHUSD':
+            return round(price * 20) / 20
+        else:
+            print('Not a valid symbole for price decimal rounding.')
+            sys.exit()
 
     def calc_pos_size(self, symbol, sl_distance):
         acc_balance = self.client.acc_balance
-        cp = self.client.last_current_price(symbol)
-        riskvalue = float(acc_balance) / 100000000 * \
-            float(cp) * (float(self._position_size)/100) # [Note]: one zero too much
-        pos_size = int(
-            round((math.floor(riskvalue / sl_distance * 5000) / 100)) * 100)
-        if pos_size < 100:
-            return 100
+        ps = self.settings[symbol].position_size
+        if symbol == 'XBTUSD':
+            btc_usd = self.client.last_current_price(symbol)
+            lot_step = 100
+            riskvalue = (acc_balance / 100000000) * \
+                btc_usd * (ps/100)  # riskvalue in usd
+            pos_size = int(
+                round((math.floor(riskvalue / (sl_distance / btc_usd)) / lot_step)) * lot_step)
+        elif symbol == 'ETHUSD':
+            tickvalue = 100
+            lot_step = 1
+            riskvalue = acc_balance * (ps/100)
+            pos_size = int(
+                round((math.floor(riskvalue / (sl_distance * tickvalue)) / lot_step)) * lot_step)
+        else:
+            print('Symbol not supported')
+            sys.exit()
+        if pos_size < lot_step:
+            return lot_step
         else:
             return pos_size
 
     def stoploss_order(self, symbol, execPrice='LastPrice'):
         try:
-            self.props[symbol].stoploss_id = self.generate_id(symbol, _type='SL')
+            self.props[symbol].stoploss_id = self.generate_id(
+                symbol, _type='SL')
             self.save_open_id(symbol, self.props[symbol].stoploss_id)
             return self.client.client.Order.Order_new(
                 symbol=self.symbol, ordType='Stop', clOrdID=self.StoplossID, orderQty=-self.props[symbol].qty,
-                stopPx=self.bitmex_decimals(self.props[symbol].SL), execInst=execPrice
+                stopPx=self.price_decimals(symbol, self.props[symbol].SL), execInst=execPrice
             ).result()
         except HTTPBadRequest as e:
             print("Error placing stoploss order.", e, flush=True)
@@ -212,7 +232,7 @@ class BitmexOrder:
             self.props[symbol].stop_id = self.generate_id(symbol)
             order_res = self.client.client.Order.Order_new(
                 symbol=symbol, ordType='Stop', clOrdID=self.props[symbol].stop_id, orderQty=orderQty,
-                stopPx=self.bitmex_decimals(entry), execInst='LastPrice'
+                stopPx=self.price_decimals(symbol, entry), execInst='LastPrice'
             ).result()
             self.props[symbol].qty = orderQty
             self.props[symbol].wait_stop = True
@@ -221,7 +241,8 @@ class BitmexOrder:
             self.props[symbol].entry = entry
             return order_res
         except HTTPBadRequest as e:
-            print("Error placing bracket stop order in {}.".format(symbol), e, flush=True)
+            print("Error placing bracket stop order in {}.".format(
+                symbol), e, flush=True)
 
     def bracket_market_order(self, symbol, orderQty, sl, tp=0):
         try:
@@ -234,7 +255,8 @@ class BitmexOrder:
             self.props[symbol].open = True
             return entry, stoploss
         except HTTPBadRequest as e:
-            print("Error placing bracket market order in {}.".format(symbol), e, flush=True)
+            print("Error placing bracket market order in {}.".format(
+                symbol), e, flush=True)
 
     def market_order(self, symbol, orderQty):
         try:
@@ -251,8 +273,8 @@ class BitmexOrder:
         if self.props[symbol].wait_stop:
             try:
                 res = self.client.client.Order.Order_amend(
-                    orderID=self.props[symbol].stoploss_id, stopPx=self.bitmex_decimals(
-                        newPrice)
+                    orderID=self.props[symbol].stoploss_id, stopPx=self.price_decimals(symbol,
+                                                                                       newPrice)
                 ).result()
                 return res
             except HTTPBadRequest as e:
